@@ -225,10 +225,10 @@ class Source(Plugin):
     __defaults = {}          # The defaults from the project
     __defaults_set = False   # Flag, in case there are not defaults at all
 
-    def __init__(self, context, project, meta, *, alias_override=None):
+    def __init__(self, context, project, meta, *, alias_override=None, unique_id=None):
         provenance = _yaml.node_get_provenance(meta.config)
         super().__init__("{}-{}".format(meta.element_name, meta.element_index),
-                         context, project, provenance, "source")
+                         context, project, provenance, "source", unique_id=unique_id)
 
         self.__element_name = meta.element_name         # The name of the element owning this source
         self.__element_index = meta.element_index       # The index of the source in the owning element's source list
@@ -585,28 +585,48 @@ class Source(Plugin):
     #
     def _fetch(self):
         project = self._get_project()
-        source_fetchers = self.get_source_fetchers()
+        context = self._get_context()
+
+        # Silence the STATUS messages which might happen as a result
+        # of checking the source fetchers.
+        with context.silence():
+            source_fetchers = self.get_source_fetchers()
 
         # Use the source fetchers if they are provided
         #
         if source_fetchers:
-            for fetcher in source_fetchers:
-                alias = fetcher._get_alias()
-                for uri in project.get_alias_uris(alias, first_pass=self.__first_pass):
-                    try:
-                        fetcher.fetch(uri)
-                    # FIXME: Need to consider temporary vs. permanent failures,
-                    #        and how this works with retries.
-                    except BstError as e:
-                        last_error = e
-                        continue
 
-                    # No error, we're done with this fetcher
-                    break
+            # Use a contorted loop here, this is to allow us to
+            # silence the messages which can result from consuming
+            # the items of source_fetchers, if it happens to be a generator.
+            #
+            source_fetchers = iter(source_fetchers)
+            try:
 
-                else:
-                    # No break occurred, raise the last detected error
-                    raise last_error
+                while True:
+
+                    with context.silence():
+                        fetcher = next(source_fetchers)
+
+                    alias = fetcher._get_alias()
+                    for uri in project.get_alias_uris(alias, first_pass=self.__first_pass):
+                        try:
+                            fetcher.fetch(uri)
+                        # FIXME: Need to consider temporary vs. permanent failures,
+                        #        and how this works with retries.
+                        except BstError as e:
+                            last_error = e
+                            continue
+
+                        # No error, we're done with this fetcher
+                        break
+
+                    else:
+                        # No break occurred, raise the last detected error
+                        raise last_error
+
+            except StopIteration:
+                pass
 
         # Default codepath is to reinstantiate the Source
         #
@@ -896,7 +916,9 @@ class Source(Plugin):
         alias = self._get_alias()
         source_kind = type(self)
 
-        clone = source_kind(context, project, self.__meta, alias_override=(alias, uri))
+        clone = source_kind(context, project, self.__meta,
+                            alias_override=(alias, uri),
+                            unique_id=self._unique_id)
 
         # Do the necessary post instantiation routines here
         #
